@@ -53,11 +53,11 @@ struct _EphyBookmark {
 static void json_serializable_iface_init (JsonSerializableIface *iface);
 static void ephy_synchronizable_iface_init (EphySynchronizableInterface *iface);
 
-G_DEFINE_TYPE_WITH_CODE (EphyBookmark, ephy_bookmark, G_TYPE_OBJECT,
-                         G_IMPLEMENT_INTERFACE (JSON_TYPE_SERIALIZABLE,
-                                                json_serializable_iface_init)
-                         G_IMPLEMENT_INTERFACE (EPHY_TYPE_SYNCHRONIZABLE,
-                                                ephy_synchronizable_iface_init))
+G_DEFINE_FINAL_TYPE_WITH_CODE (EphyBookmark, ephy_bookmark, G_TYPE_OBJECT,
+                               G_IMPLEMENT_INTERFACE (JSON_TYPE_SERIALIZABLE,
+                                                      json_serializable_iface_init)
+                               G_IMPLEMENT_INTERFACE (EPHY_TYPE_SYNCHRONIZABLE,
+                                                      ephy_synchronizable_iface_init))
 
 enum {
   PROP_0,
@@ -464,32 +464,39 @@ int
 ephy_bookmark_bookmarks_compare_func (EphyBookmark *bookmark1,
                                       EphyBookmark *bookmark2)
 {
-  gint64 time1;
-  gint64 time2;
-  const char *title1;
-  const char *title2;
-  int title_result;
+  g_autofree char *title1 = NULL;
+  g_autofree char *title2 = NULL;
+  int result;
   const char *url1;
   const char *url2;
+  gint64 time1;
+  gint64 time2;
 
   g_assert (EPHY_IS_BOOKMARK (bookmark1));
   g_assert (EPHY_IS_BOOKMARK (bookmark2));
 
-  time1 = ephy_bookmark_get_time_added (bookmark1);
-  time2 = ephy_bookmark_get_time_added (bookmark2);
-  if (time2 - time1 != 0)
-    return time2 - time1;
+  if (ephy_bookmark_has_tag (bookmark1, EPHY_BOOKMARKS_FAVORITES_TAG) &&
+      !ephy_bookmark_has_tag (bookmark2, EPHY_BOOKMARKS_FAVORITES_TAG))
+    return -1;
+  if (!ephy_bookmark_has_tag (bookmark1, EPHY_BOOKMARKS_FAVORITES_TAG) &&
+      ephy_bookmark_has_tag (bookmark2, EPHY_BOOKMARKS_FAVORITES_TAG))
+    return 1;
 
-  title1 = ephy_bookmark_get_title (bookmark1);
-  title2 = ephy_bookmark_get_title (bookmark2);
-  title_result = g_strcmp0 (title1, title2);
-  if (title_result != 0)
-    return title_result;
+  title1 = g_utf8_casefold (ephy_bookmark_get_title (bookmark1), -1);
+  title2 = g_utf8_casefold (ephy_bookmark_get_title (bookmark2), -1);
+  result = g_strcmp0 (title1, title2);
+  if (result != 0)
+    return result;
 
   url1 = ephy_bookmark_get_url (bookmark1);
   url2 = ephy_bookmark_get_url (bookmark2);
+  result = g_strcmp0 (url1, url2);
+  if (result != 0)
+    return result;
 
-  return g_strcmp0 (url1, url2);
+  time1 = ephy_bookmark_get_time_added (bookmark1);
+  time2 = ephy_bookmark_get_time_added (bookmark2);
+  return time2 - time1;
 }
 
 int
@@ -497,11 +504,14 @@ ephy_bookmark_tags_compare (const char *tag1,
                             const char *tag2)
 {
   int result;
+  int result_casefolded;
 
   g_assert (tag1 != NULL);
   g_assert (tag2 != NULL);
 
   result = g_strcmp0 (tag1, tag2);
+  result_casefolded = g_strcmp0 (g_utf8_casefold (tag1, -1),
+                                 g_utf8_casefold (tag2, -1));
 
   if (result == 0)
     return 0;
@@ -511,7 +521,10 @@ ephy_bookmark_tags_compare (const char *tag1,
   if (g_strcmp0 (tag2, EPHY_BOOKMARKS_FAVORITES_TAG) == 0)
     return 1;
 
-  return result;
+  if (result_casefolded == 0)
+    return result;
+
+  return result_casefolded;
 }
 
 char *
